-- E.g. { "bm_suit_jumpsuit", "bm_suit_miami" }
heist_outfit_selector.outfit_localization_ids = {}
-- E.g. { ["bm_suit_jumpsuit"] = "jumpsuit", ["bm_suit_miami"] = "miami" }
heist_outfit_selector.outfits_by_localization_id = {}

-- E.g. { "bm_gloves_heatleather", "bm_gloves_sneak" }
heist_outfit_selector.glove_localization_ids = {}
-- E.g. { ["bm_gloves_heatleather"] = "heat", ["bm_gloves_sneak"] = "sneak" }
heist_outfit_selector.gloves_by_localization_id = {}

-- E.g. { ["firestarter_1"] = "heist_firestarter" }
heist_outfit_selector.job_name_ids_by_level_id = {}

-- These heists appear in the game files, but are unused/unimplemented, hence we need to exclude them ourselves
local level_blocklist = {
	"mia2_new",
	"driving_escapes_industry_day",
	"driving_escapes_city_day",
}

-- Heists whose names should never be prepended to level names
local job_prefix_blocklist = {
	-- Otherwise, we get "Bank Heist: Gold: Bank Heist"
	"heist_branchbank",
	"heist_branchbank_deposit",
	"heist_branchbank_cash",
	"heist_branchbank_gold",
	-- Otherwise, we get "No Mercy: No Mercy (Flashback)"
	"heist_nmh",
}

-- Heists whose names should always be prepended to level names, even if the level name is the same
local job_prefix_allowlist = {
	-- Makes day 1 show up as "Hotline Miami: Hotline Miami", to stay consistent with day 2 ("Hotline Miami: Four Floors")
	"heist_mia",
}

function heist_outfit_selector:initialize_default_heist_outfit_options()
	heist_outfit_selector:initialize_outfits()
	heist_outfit_selector:initialize_job_names()

	return heist_outfit_selector:build_levels_table()
end

function heist_outfit_selector:initialize_default_outfit_glove_options()
	heist_outfit_selector:initialize_gloves()

	return heist_outfit_selector:build_outfits_table()
end

function heist_outfit_selector:initialize_outfits()
	local function init_outfit(outfit_style_id, outfit_localization_id)
		table.insert(heist_outfit_selector.outfit_localization_ids, outfit_localization_id)
		heist_outfit_selector.outfits_by_localization_id[outfit_localization_id] = outfit_style_id
	end

	init_outfit("heist_default", "HeistOutfitSelector_heist_default")
	init_outfit("character_default", "HeistOutfitSelector_character_default")

	for style_id, style_data in pairs(tweak_data.blackmarket.player_styles) do
		--! one of them does not have the name_id variable and breaks the mod initialization
		if style_id ~= nil and style_id ~= "none" and style_data.name_id then
			init_outfit(style_id, style_data.name_id)
		end
	end
end

function heist_outfit_selector:initialize_gloves()
	local function init_glove(glove_id, glove_localization_id)
		table.insert(heist_outfit_selector.glove_localization_ids, glove_localization_id)
		heist_outfit_selector.gloves_by_localization_id[glove_localization_id] = glove_id
	end

	init_glove("outfit_default", "HeistOutfitSelector_outfit_default")
	init_glove("character_default", "HeistOutfitSelector_character_default")
	init_glove("heist_default", "HeistOutfitSelector_original_blue_gloves")

	for glove_id, glove_data in pairs(tweak_data.blackmarket.gloves) do
		if glove_id ~= nil and glove_id ~= "default" and glove_id ~= "heist_default" then
			init_glove(glove_id, glove_data.name_id)
		end
	end
end

--[[
    Initialise a map of level ID to job name ID. This map is used in build_levels_table() in order to prepend each
    level name with the name of the heist it belongs to (e.g. "Firestarter: Airport").
]]
function heist_outfit_selector:initialize_job_names()
	local function init_job(level_id, job_name_id)
		heist_outfit_selector.job_name_ids_by_level_id[level_id] = job_name_id
	end

	for index, job_id in ipairs(tweak_data.narrative:get_jobs_index()) do
		local job_data = tweak_data.narrative:job_data(job_id)
		local job_chain = tweak_data.narrative:job_chain(job_id)
		local job_name_id = job_data.name_id

		for index, chain_stage in ipairs(job_chain) do
			local level_id = chain_stage.level_id
			if level_id ~= nil then
				init_job(level_id, job_name_id)
			else
				-- Sometimes (like Election Day), stages are weirdly doubly nested
				for index, chain_nested_stage in ipairs(chain_stage) do
					local level_id = chain_nested_stage.level_id
					if level_id ~= nil then
						init_job(level_id, job_name_id)
					end
				end
			end
		end
	end
end

function heist_outfit_selector:build_levels_table()
	local function build_level_prefix(level_id, level_name, job_name_id, job_name)
		if string.match(level_id, "^short[0-9]+_.*") then
			return "Tutorial: "
		elseif string.match(level_id, "^skm_.*") then
			return "Holdout: "
		end

		if
			job_name == nil
			or job_name == ""
			or table.contains(job_prefix_blocklist, job_name_id)
			or (job_name == level_name and not table.contains(job_prefix_allowlist, job_name_id))
		then
			return ""
		else
			return job_name .. ": "
		end
	end

	local function build_level_suffix(level_id)
		if string.ends(level_id, "_night") then
			return " Night"
		elseif string.ends(level_id, "_day") then
			return " Day"
		else
			return ""
		end
	end

	local levels_table = {}
	for _, level_id in pairs(tweak_data.levels._level_index) do
		if tweak_data.levels[level_id] ~= nil and not table.contains(level_blocklist, level_id) then
			table.insert(levels_table, {
				name = level_id,
				title_id = function()
					local level_name_id = tweak_data.levels[level_id].name_id
					local level_name = managers.localization:exists(level_name_id)
							and managers.localization:text(level_name_id)
						or level_id

					local job_name_id = heist_outfit_selector.job_name_ids_by_level_id[level_id]
					local job_name = job_name_id ~= nil
							and managers.localization:exists(job_name_id)
							and managers.localization:text(job_name_id)
						or ""

					local prefix = build_level_prefix(level_id, level_name, job_name_id, job_name)
					local suffix = build_level_suffix(level_id)

					return prefix .. level_name .. suffix
				end,
				default_value = 1,
			})
		end
	end

	return levels_table
end

function heist_outfit_selector:build_outfits_table()
	local outfits_table = {}

	local function build_outfit(outfit_id, outfit_data)
		table.insert(outfits_table, {
			name = outfit_id,
			title_id = function()
				local outfit_name_id = outfit_data.name_id
				return managers.localization:exists(outfit_name_id) and managers.localization:text(outfit_name_id)
					or outfit_name_id
			end,
			default_value = 1,
		})
	end

	-- Ensure that the default outfit appears first in the list
	build_outfit("none", tweak_data.blackmarket.player_styles["none"])

	for outfit_id, outfit_data in pairs(tweak_data.blackmarket.player_styles) do
		--! one of them does not have the name_id variable and crashes the game
		if outfit_id ~= "none" and outfit_data.name_id then
			build_outfit(outfit_id, outfit_data)
		end
	end

	return outfits_table
end

--[[
    In the mod's options file, outfits are saved by their localization ID (e.g. "bm_suit_jumpsuit"),
    rather than their outfit ID (e.g. "jumpsuit").

    This is so that they can be programmatically localized when viewed in the mod's options menu.

    However, it means that we need to convert the localization ID back into the outfit ID before we apply the outfit
    to a given heist.

    There might be a better way to handle the persistence of these values, which wouldn't require converting back and
    forth, but I'm not familiar enough with the library. I think what I would be looking for is a way to pass a
    populate_items function into <item_parameters>, the same way we can for <option_set>. But instead it seems we can
    only pass in a simple predefined table.
]]
function heist_outfit_selector:get_outfit_for_level(level_id)
	local outfit_localization_id = heist_outfit_selector.Options:GetValue("DefaultHeistOutfits/" .. level_id, true)
	if outfit_localization_id ~= nil then
		return heist_outfit_selector.outfits_by_localization_id[outfit_localization_id]
	else
		return nil
	end
end

--[[
    As outlined above get_outfit_for_level(), gloves are stored by localization ID.
]]
function heist_outfit_selector:get_glove_for_outfit(outfit_id)
	local glove_localization_id = heist_outfit_selector.Options:GetValue("DefaultOutfitGloves/" .. outfit_id, true)
	if glove_localization_id ~= nil then
		return heist_outfit_selector.gloves_by_localization_id[glove_localization_id]
	else
		return nil
	end
end

function heist_outfit_selector:should_include_modded_outfit_variations()
	local value = heist_outfit_selector.Options:GetValue("ShouldIncludeModdedOutfitVariations")
	if value == nil then
		return false
	else
		return value
	end
end

function heist_outfit_selector:should_use_unique_outfit_variations()
	local value = heist_outfit_selector.Options:GetValue("OutfitVariations", true)
	if value == nil then
		return true
	else
		return value == "ShouldUseUniqueOutfitVariations"
	end
end

function heist_outfit_selector:should_use_matching_outfit_variations()
	local value = heist_outfit_selector.Options:GetValue("OutfitVariations", true)
	if value == nil then
		return false
	else
		return value == "ShouldUseMatchingOutfitVariations"
	end
end

function heist_outfit_selector:should_use_default_outfit_variation()
	local value = heist_outfit_selector.Options:GetValue("OutfitVariations", true)
	if value == nil then
		return false
	else
		return value == "ShouldUseDefaultOutfitVariation"
	end
end

function heist_outfit_selector:should_override_own_outfit()
	local value = heist_outfit_selector.Options:GetValue("ShouldOverrideOwnOutfitSelections", true)
	if value == nil then
		return false
	else
		return value
	end
end

function heist_outfit_selector:should_override_crew_outfits()
	local value = heist_outfit_selector.Options:GetValue("ShouldOverrideCrewOutfitSelections", true)
	if value == nil then
		return false
	else
		return value
	end
end
